@@ -1,5 +1,27 @@
-#!/usr/bin/env python
-#-*- coding: utf-8 -*-
+# coding: utf-8
+#
+#    Project: X-ray image reader
+#             https://github.com/kif/fabio
+#
+#
+#    Copyright (C) European Synchrotron Radiation Facility, Grenoble, France
+#
+#    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+
 """
 FabIO class for dealing with TIFF images.
 In facts wraps TiffIO from V. Armando Solé (available in PyMca) or falls back to PIL
@@ -22,7 +44,7 @@ License: GPLv3+
 from __future__ import with_statement, print_function, division
 
 __authors__ = ["Jérôme Kieffer", "Henning O. Sorensen", "Erik Knudsen"]
-__date__ = "11/07/2011"
+__date__ = "03/12/2015"
 __license__ = "GPLv3+"
 __copyright__ = "ESRF, Grenoble & Risoe National Laboratory"
 __status__ = "stable"
@@ -34,16 +56,33 @@ except ImportError:
     logger.warning("PIL is not installed ... trying to do without")
     Image = None
 import numpy
-from .fabioimage import fabioimage
-from .TiffIO     import TiffIO
+from .fabioimage import FabioImage
 
-PIL_TO_NUMPY = { "I;16": numpy.uint16,
-                   "F": numpy.float32,
-                   "1": numpy.bool,
-                   "I": numpy.int32,
-                   "L": numpy.uint8,
-                    }
+try:
+    from PyMca.TiffIO import TiffIO
+except ImportError:
+    try:
+        from PyMca5.PyMcaIO.TiffIO import TiffIO
+    except ImportError:
+        from .third_party.TiffIO import TiffIO
 
+
+PIL_TO_NUMPY = {"I;8": numpy.uint8,
+                "I;16": numpy.uint16,
+                "I;16B": numpy.uint16,  # big endian
+                "I;16L": numpy.uint16,  # little endian
+                "I;32":numpy.uint32,
+                "I;32L":numpy.uint32,  # little endian
+                "I;32B":numpy.uint32,  # big endian
+                "F;32F":numpy.float32,
+                "F;32BF":numpy.float32,  # big endian
+                "F;64F":numpy.float64,
+                "F;64BF": numpy.float64,  # big endian
+                "F": numpy.float32,
+                "1": numpy.bool,
+                "I": numpy.int32,
+                "L": numpy.uint8,
+                }
 LITTLE_ENDIAN = 1234
 BIG_ENDIAN = 3412
 
@@ -79,7 +118,8 @@ baseline_tiff_tags = {
   33432:'Copyright'
   }
 
-class tifimage(fabioimage):
+
+class TifImage(FabioImage):
     """
     Images in TIF format
     Wraps TiffIO
@@ -89,7 +129,7 @@ class tifimage(fabioimage):
     def __init__(self, *args, **kwds):
         """ Tifimage constructor adds an nbits member attribute """
         self.nbits = None
-        fabioimage.__init__(self, *args, **kwds)
+        FabioImage.__init__(self, *args, **kwds)
         self.lib = None
 
     def _readheader(self, infile):
@@ -112,8 +152,8 @@ class tifimage(fabioimage):
         header = numpy.fromstring(infile.read(64), numpy.uint16)
         self.dim1 = int(header[9])
         self.dim2 = int(header[15])
-#         nbits is not a fabioimage attribute...
-        self.nbits = int(header[21]) # number of bits
+#         nbits is not a FabioImage attribute...
+        self.nbits = int(header[21])  # number of bits
 
     def read(self, fname, frame=None):
         """
@@ -126,7 +166,7 @@ class tifimage(fabioimage):
         try:
             tiffIO = TiffIO(infile)
             if tiffIO.getNumberOfImages() > 0:
-                #No support for now of multi-frame tiff images
+                # No support for now of multi-frame tiff images
                 self.data = tiffIO.getImage(0)
                 self.header = tiffIO.getInfo(0)
         except Exception as error:
@@ -152,25 +192,37 @@ class tifimage(fabioimage):
                     infile.seek(0)
                 else:
                     self.lib = "PIL"
-                    self.dim1, self.dim2 = self.pilimage.size
+                    dim1, dim2 = self.pilimage.size
                     if self.pilimage.mode in PIL_TO_NUMPY:
-                        self.data = numpy.fromstring(self.pilimage.tostring(), PIL_TO_NUMPY[self.pilimage.mode])
-                    else: #probably RGB or RGBA images: rely on PIL to convert it to greyscale float.
-                        self.data = numpy.fromstring(self.pilimage.convert("F").tostring(), numpy.float32)
-                    self.data.shape = (self.dim2, self.dim1)
+                        dtype = PIL_TO_NUMPY[self.pilimage.mode]
+                        pilimage = self.pilimage
+                    else:
+                        dtype = numpy.float32
+                        pilimage = self.pilimage.convert("F")
+                    try:
+                        data = numpy.asarray(pilimage, dtype)
+                    except:  # PIL does not support buffer interface (yet)
+                        if "tobytes" in dir(pilimage):
+                            data = numpy.fromstring(pilimage.tobytes(), dtype=dtype)
+                        else:
+                            data = numpy.fromstring(pilimage.tostring(), dtype=dtype)
+                        # byteswap ?
+                        if numpy.dtype(dtype).itemsize > 1:
+                            if (numpy.little_endian and "B" in self.pilimage.mode) or\
+                               (not numpy.little_endian and self.pilimage.mode.endswith("L")):
+                                data.byteswap(True)
+                    self.data = data.reshape((dim2, dim1))
             else:
                 logger.error("Error in opening %s: no tiff reader managed to read the file.", fname)
                 self.lib = None
                 infile.seek(0)
 
-        self.bpp = len(numpy.zeros(1, dtype=self.data.dtype).tostring())
-        self.bytecode = self.data.dtype.name
         self.resetvals()
         return self
 
     def write(self, fname):
         """
-        Overrides the fabioimage.write method and provides a simple TIFF image writer.
+        Overrides the FabioImage.write method and provides a simple TIFF image writer.
         @param fname: name of the file to save the image to
         @tag_type fname: string or unicode (file?)...
         """
@@ -178,8 +230,7 @@ class tifimage(fabioimage):
         tiffIO.writeImage(self.data, info=self.header, software="fabio.tifimage", date=time.ctime())
 
 
-
-#define a couple of helper classes here:
+# define a couple of helper classes here:
 class Tiff_header(object):
     def __init__(self, string):
         if string[:4] == "II\x2a\x00":
@@ -188,7 +239,7 @@ class Tiff_header(object):
             self.byteorder = BIG_ENDIAN
         else:
             logger.warning("Warning: This does not appear to be a tiff file")
-        #the next two bytes contains the offset of the oth IFD
+        # the next two bytes contains the offset of the oth IFD
         offset_first_ifd = struct.unpack_from("h", string[4:])[0]
         self.ifd = [Image_File_Directory()]
         offset_next = self.ifd[0].unpack(string, offset_first_ifd)
@@ -197,7 +248,7 @@ class Tiff_header(object):
             offset_next = self.ifd[-1].unpack(string, offset_next)
 
         self.header = {}
-        #read the values of the header items into a dictionary
+        # read the values of the header items into a dictionary
         for entry in self.ifd[0].entries:
             if entry.tag in baseline_tiff_tags.keys():
                 self.header[baseline_tiff_tags[entry.tag]] = entry.val
@@ -205,26 +256,26 @@ class Tiff_header(object):
                 self.header[entry.tag] = entry.val
 
 class Image_File_Directory(object):
-    def __init__(self, instring=None, offset= -1):
+    def __init__(self, instring=None, offset=-1):
         self.entries = []
         self.offset = offset
         self.count = None
 
-    def unpack(self, instring, offset= -1):
+    def unpack(self, instring, offset=-1):
         if (offset == -1): offset = self.offset
 
         strInput = instring[offset:]
         self.count = struct.unpack_from("H", strInput[:2])[0]
-        #0th IFD contains count-1 entries (count includes the adress of the next IFD)
+        # 0th IFD contains count-1 entries (count includes the adress of the next IFD)
         for i in range(self.count - 1):
             e = Image_File_Directory_entry().unpack(strInput[2 + 12 * (i + 1):])
             if (e != None):
                 self.entries.append(e)
-            #extract data associated with tags
+            # extract data associated with tags
             for e in self.entries:
                 if (e.val == None):
                     e.extract_data(instring)
-        #do we have some more ifds in this file
+        # do we have some more ifds in this file
         offset_next = struct.unpack_from("L", instring[offset + 2 + self.count * 12:])[0]
         return offset_next
 
@@ -289,3 +340,6 @@ class Image_File_Directory_entry(object):
                 self.val = struct.unpack_from("d", full_string[self.val_offset:])[0]
         else:
             logger.warning("unrecognized type of strInputentry self: %s tag: %s type: %s TYPE: %s" % (self, baseline_tiff_tags[self.tag], self.tag_type, TYPES[tag_type]))
+
+
+tifimage = TifImage
